import { Server } from 'socket.io'
import { TSocket } from '#utilities/types'
import prisma from '#utilities/prisma'
import CharacterRepository from '#repositories/characterRepository'
import { gameMasterLogger } from '#utilities/logger'
import type { Prisma } from '@prisma/client'

interface CopyPayload {
  id: string
}

export default class SpriteCopyEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('gm:sprite:copy', this.handleSpriteCopy.bind(this))
  }

  private async handleSpriteCopy(payload: CopyPayload, callback: (success: boolean) => void): Promise<void> {
    try {
      if (!(await this.validateGameMasterAccess())) {
        return callback(false)
      }

      const sourceSprite = await prisma.sprite.findUnique({
        where: { id: payload.id },
        include: {
          spriteActions: true
        }
      })

      if (!sourceSprite) {
        throw new Error('Source sprite not found')
      }

      const newSprite = await prisma.sprite.create({
        data: {
          name: `${sourceSprite.name} (Copy)`,
          spriteActions: {
            create: sourceSprite.spriteActions.map((action) => ({
              action: action.action,
              sprites: action.sprites as Prisma.InputJsonValue,
              originX: action.originX,
              originY: action.originY,
              isAnimated: action.isAnimated,
              isLooping: action.isLooping,
              frameWidth: action.frameWidth,
              frameHeight: action.frameHeight,
              frameRate: action.frameRate
            }))
          }
        }
      })

      callback(true)
    } catch (error) {
      this.handleError(error, payload.id, callback)
    }
  }

  private async validateGameMasterAccess(): Promise<boolean> {
    const character = await CharacterRepository.getById(this.socket.characterId!)
    return character?.role === 'gm'
  }

  private handleError(error: unknown, spriteId: string, callback: (success: boolean) => void): void {
    gameMasterLogger.error(`Error copying sprite ${spriteId}: ${this.getErrorMessage(error)}`)
    callback(false)
  }

  private getErrorMessage(error: unknown): string {
    return error instanceof Error ? error.message : String(error)
  }
}
