import config from '../../mikro-orm.config'
// import { MikroORM } from '@mikro-orm/mariadb'
import { MikroORM } from '@mikro-orm/mysql'
import { appLogger } from '#utilities/logger'

/**
 * Singleton class for initializing and managing the database connection
 */
export class Database {
  private static instance: MikroORM | undefined

  private static async init(): Promise<MikroORM> {
    try {
      const orm = await MikroORM.init(config)
      appLogger.info('MikroORM connected successfully')
      return orm
    } catch (error) {
      appLogger.error(`MikroORM connection failed: ${error}`)
      throw error
    }
  }

  public static async getInstance(): Promise<MikroORM> {
    if (!Database.instance) {
      Database.instance = await Database.init()
    }
    return Database.instance
  }

  public static async save<T extends object>(entity: T): Promise<T> {
    try {
      const orm = await Database.getInstance()
      const em = orm.em.fork() // Create a new EntityManager instance

      await em.begin() // Start transaction
      try {
        em.persist(entity)
        await em.flush()
        await em.commit()
        return entity
      } catch (error) {
        await em.rollback()
        throw error
      }
    } catch (error) {
      appLogger.error(`Failed to save entity: ${error instanceof Error ? error.message : String(error)}`)
      throw error
    }
  }

  public static async delete<T extends object>(entity: T): Promise<T> {
    try {
      const orm = await Database.getInstance()
      const em = orm.em.fork() // Create a new EntityManager instance

      await em.begin() // Start transaction
      try {
        em.remove(entity)
        await em.flush()
        await em.commit()
        return entity
      } catch (error) {
        await em.rollback()
        throw error
      }
    } catch (error) {
      appLogger.error(`Failed to remove entity: ${error instanceof Error ? error.message : String(error)}`)
      throw error
    }
  }

  public static async close() {
    if (Database.instance) {
      await Database.instance.close()
      Database.instance = undefined
    }
  }
}
